"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compile = exports.transformRootLevel = exports.transformMethodLevel = exports.transformSchemaLevel = exports.transformResponse = void 0;
const json_schema_to_typescript_1 = require("json-schema-to-typescript");
const constants_1 = require("./constants");
const utils_1 = require("./utils");
const transformResponse = (title, properties) => Object.keys(properties).map((key) => {
    const item = properties[key];
    if (item['$ref']) {
        return item;
    }
    return {
        ...item,
        title: `${title}Status${item.title ? (0, utils_1.normalizeTitle)(item.title) : key}`,
    };
});
exports.transformResponse = transformResponse;
const transformSchemaLevel = (title, schema, list) => {
    const result = list.reduce((acc, { name, capitalized }) => {
        const properties = schema[name];
        const newTitle = (properties === null || properties === void 0 ? void 0 : properties.title) || title + capitalized;
        if (properties) {
            acc[capitalized] =
                name === 'response'
                    ? {
                        ...(0, utils_1.createSchemaObject)(newTitle, undefined),
                        oneOf: (0, exports.transformResponse)(newTitle, properties),
                    }
                    : { ...properties, title: newTitle };
        }
        return acc;
    }, {});
    return Object.keys(result).length ? result : undefined;
};
exports.transformSchemaLevel = transformSchemaLevel;
const transformMethodLevel = (title, config, ignoreHead) => config.reduce((acc, record) => {
    const methods = typeof record.method === 'string' ? [record.method] : record.method;
    methods.forEach((method) => {
        if (!ignoreHead || method !== 'HEAD') {
            const newTitle = title + (0, utils_1.capitalize)(method);
            const schema = record.schema || {};
            const list = constants_1.methodsWithBody.includes(method)
                ? constants_1.cachedSchemasWithBody
                : constants_1.cachedSchemas;
            const properties = (0, exports.transformSchemaLevel)(newTitle, schema, list);
            acc[method] = {
                ...(0, utils_1.createSchemaObject)(newTitle, properties),
                description: schema.description,
            };
        }
    });
    return acc;
}, {});
exports.transformMethodLevel = transformMethodLevel;
const transformRootLevel = (routes, ignoreHead) => routes.reduce((acc, [route, config]) => {
    const title = (0, utils_1.generateEndpointName)(route);
    const properties = (0, exports.transformMethodLevel)(title, config, ignoreHead);
    acc[route] = (0, utils_1.createSchemaObject)(undefined, properties);
    return acc;
}, {});
exports.transformRootLevel = transformRootLevel;
const compile = async (routes, definitions, ignoreHead, compilerOptions = {}) => {
    const options = (0, utils_1.patchCompilerOptions)(compilerOptions, definitions);
    const updatedRoutes = Array.from(routes).sort(([a], [b]) => a < b ? -1 : a > b ? 1 : 0);
    const properties = (0, exports.transformRootLevel)(updatedRoutes, ignoreHead);
    const schema = { ...(0, utils_1.createSchemaObject)(constants_1.rootName, properties), definitions };
    const text = await (0, json_schema_to_typescript_1.compile)(schema, constants_1.rootName, options);
    return {
        text: text
            .replace(/\}\n\//g, '}\n\n/')
            .replace(/\}\nexport /g, '}\n\nexport ')
            .replace(/export type/g, '\nexport type'),
        schema,
    };
};
exports.compile = compile;
