"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.patchCompilerOptions = exports.createSchemaObject = exports.generateEndpointName = exports.normalizeTitle = exports.capitalize = void 0;
const generated = {};
const calcCwd = (cwd) => process.platform === 'win32' ? cwd.replace(/\\/g, '/') : cwd;
const capitalize = (str) => { var _a; return ((_a = str[0]) !== null && _a !== void 0 ? _a : '').toUpperCase() + str.substring(1).toLowerCase(); };
exports.capitalize = capitalize;
const normalizeTitle = (title) => title
    .replace(/[^0-9a-z ]/gi, ' ')
    .split(' ')
    .filter((item) => item !== '')
    .map((item) => item[0].toUpperCase() + item.slice(1))
    .join('');
exports.normalizeTitle = normalizeTitle;
const generateEndpointName = (endpoint) => {
    if (!endpoint) {
        return 'Empty';
    }
    const parts = endpoint
        .replace('/*', '/all')
        .replace(/[^0-9a-z/?&=:-]/gi, '')
        .split(/[/?&=:-]+/)
        .filter((item) => item !== '')
        .map((item) => item[0].toUpperCase() + item.slice(1));
    if (!parts.length) {
        return 'Root';
    }
    const name = parts.join('');
    const counter = generated[name] || 0;
    generated[name] = counter + 1;
    return counter ? name + counter : name;
};
exports.generateEndpointName = generateEndpointName;
const createSchemaObject = (title, properties) => ({
    title,
    type: 'object',
    required: properties ? Object.keys(properties) : undefined,
    properties,
    additionalProperties: false,
});
exports.createSchemaObject = createSchemaObject;
const patchCompilerOptions = (options, definitions) => {
    var _a;
    const cwd = calcCwd(options.cwd || process.cwd());
    return {
        ...options,
        bannerComment: '',
        format: false,
        $refOptions: {
            ...options.$refOptions,
            resolve: {
                file: {
                    read: (file) => {
                        const key = file.url.substring(cwd.length);
                        const text = definitions[key] || definitions[key.slice(1)];
                        if (!text) {
                            const error = new Error(`fastify-extract-definitions: cannot resolve schema for ${file.url}`);
                            throw error;
                        }
                        return text;
                    },
                },
                ...(_a = options.$refOptions) === null || _a === void 0 ? void 0 : _a.resolve,
            },
        },
    };
};
exports.patchCompilerOptions = patchCompilerOptions;
